LPDIRECTINPUT8 dinput;
LPDIRECTINPUTDEVICE8 di_mouse[50];
LPDIRECTINPUTDEVICE8 di_keyboard;

DIMOUSESTATE di_mousestate;
BYTE di_keystate[256];

int di_mouse_count;
char di_mouse_guid[50][512];
HWND di_enum_hwnd;


#define DIMOUSE_LEFTBUTTON   0
#define DIMOUSE_RIGHTBUTTON  1
#define DIMOUSE_MIDDLEBUTTON 2
#define DIMOUSE_4BUTTON      3


// From WINE
void guid_to_string( const GUID* guid, char *str )
{
	sprintf(str, "{%08x-%04x-%04x-%02X%02X-%02X%02X%02X%02X%02X%02X}",
		guid->Data1, guid->Data2, guid->Data3,
		guid->Data4[0], guid->Data4[1], guid->Data4[2], guid->Data4[3],
		guid->Data4[4], guid->Data4[5], guid->Data4[6], guid->Data4[7]);
}



// NOTE: XP only sees 'SysMouse'
BOOL CALLBACK DInput_EnumCallback( const DIDEVICEINSTANCE* instance, VOID* context )
{
	if( FAILED( dinput->CreateDevice( instance->guidInstance, &di_mouse[ di_mouse_count ], NULL )) )
		return DIENUM_CONTINUE;

	if( FAILED( di_mouse[ di_mouse_count ]->SetDataFormat( &c_dfDIMouse )) )
		return DIENUM_CONTINUE;

	if( FAILED( di_mouse[ di_mouse_count ]->SetCooperativeLevel( di_enum_hwnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND )) )
		return DIENUM_CONTINUE;


	// save name
	guid_to_string( &instance->guidInstance, di_mouse_guid[ di_mouse_count ] );


	di_mouse_count++;
}


void Init_DInput( HINSTANCE hInstance, HWND hWnd )
{
	di_enum_hwnd = hWnd;


	if( FAILED( DirectInput8Create( hInstance, DIRECTINPUT_VERSION, IID_IDirectInput8,
		(void**) &dinput, NULL)) ) {
		return;
	}


	if( hidmouse_api == HIDMOUSE_API_DIRECTINPUT )
	{
		di_mouse_count = 0;

		dinput->EnumDevices( DI8DEVCLASS_POINTER, DInput_EnumCallback, NULL, DIEDFL_ATTACHEDONLY );
	}



	if( FAILED( dinput->CreateDevice( GUID_SysKeyboard, &di_keyboard, NULL)) )
		return;

	if( FAILED( di_keyboard->SetDataFormat( &c_dfDIKeyboard )) )
		return;

	if( FAILED( di_keyboard->SetCooperativeLevel( hWnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND )) )
		return;
}


void Poll_Mouse_DInput()
{
	HRESULT hr;

	if( !di_mouse[0] ) return;
	if( hidmouse_api != HIDMOUSE_API_DIRECTINPUT ) return;



	for( int lcv = 0; lcv < di_mouse_count; lcv++ )
	{
		device_x[ lcv ] = 0;
		device_y[ lcv ] = 0;
		device_z[ lcv ] = 0;


		device_buttons[ lcv ][0] = 0;
		device_buttons[ lcv ][1] = 0;
		device_buttons[ lcv ][2] = 0;


		// assume lost input (alt-tab)
		hr = DIERR_INPUTLOST;
		while( hr == DIERR_INPUTLOST )
			hr = di_mouse[ lcv ]->Acquire();
		if( FAILED(hr) ) return;


		hr = di_mouse[ lcv ]->GetDeviceState( sizeof(DIMOUSESTATE), (LPVOID)&di_mousestate );
		if( FAILED(hr) ) return;


		device_x[ lcv ] = di_mousestate.lX;
		device_y[ lcv ] = di_mousestate.lY;
		device_z[ lcv ] = di_mousestate.lZ;


		device_buttons[ lcv ][0] = 0;
		device_buttons[ lcv ][1] = 0;
		device_buttons[ lcv ][2] = 0;


		if( di_mousestate.rgbButtons[ DIMOUSE_LEFTBUTTON ] & 0x80 )
			device_buttons[ lcv ][0] = 1;
		if( di_mousestate.rgbButtons[ DIMOUSE_RIGHTBUTTON ] & 0x80 )
			device_buttons[ lcv ][1] = 1;
		if( di_mousestate.rgbButtons[ DIMOUSE_MIDDLEBUTTON ] & 0x80 )
			device_buttons[ lcv ][2] = 1;
	}

	
	device_absolute[0] = 0;
	device_absolute[1] = 0;
}


void Poll_Keyboard_DInput()
{
	HRESULT hr;

	if( !di_keyboard ) return;


	di_keyboard->Acquire();
	hr = di_keyboard->GetDeviceState( 256, (LPVOID)di_keystate );

	if( FAILED(hr) )
	{
		memset( save.state[0], 0, sizeof(save.state[0]) );
		memset( save.state[1], 0, sizeof(save.state[1]) );


		while( hr == DIERR_INPUTLOST )
			hr = di_keyboard->Acquire();

		if( FAILED(hr) ) return;


		di_keyboard->GetDeviceState( 256, (LPVOID)di_keystate );
	}
}


void Done_DInput()
{
	for( int lcv = 0; lcv < di_mouse_count; lcv++ ) {
		if( di_mouse[ lcv ] ) {
			di_mouse[ lcv ]->Unacquire();
			di_mouse[ lcv ]->Release();
			di_mouse[ lcv ] = NULL;
		}
	}


	if( di_keyboard ) {
		di_keyboard->Unacquire();
		di_keyboard->Release();
		di_keyboard = NULL;
	}


	if( dinput ) {
		dinput->Release();
		dinput = NULL;
	}


	di_mouse_count = 0;
}
